@@ -24,11 +24,8 @@
 #include <KAction>
 #include <KActionCollection>
 #include <KLocale>
-#include <KApplication>
-#include <KAction>
-#include <KLocale>
-#include <KActionCollection>
 #include <KAboutData>
+#include <KFilterProxySearchLine>
 #include <KFindDialog>
 #include <KStandardAction>
 #include <KService>
@@ -39,6 +36,7 @@
 #include <KNotifyConfigWidget>
 #include <KPasswordDialog>
 #include <QTextCodec>
+#include <QClipboard>
 #include <QGridLayout>
 #include <QLabel>
 #include <QLineEdit>
@@ -50,10 +48,15 @@
 #include <CallList.h>
 #include <FritzClient.h>
 
+#include "ContainerWidget.h"
+#include "DialDialog.h"
+#include "KCalllistProxyModel.h"
 #include "KSettings.h"
 #include "KSettingsFonbooks.h"
 #include "KSettingsFritzBox.h"
+#include "KSettingsMisc.h"
 #include "Log.h"
+#include "MimeFonbookEntry.h"
 
 KFritzWindow::KFritzWindow()
 {
@@ -72,7 +75,6 @@ KFritzWindow::KFritzWindow()
 	fritz::Config::SetupLogging(LogStream::getLogStream(LogBuf::DEBUG)->setLogWidget(logArea),
 							    LogStream::getLogStream(LogBuf::INFO)->setLogWidget(logArea),
 					            LogStream::getLogStream(LogBuf::ERROR)->setLogWidget(logArea));
-//TODO: support multiple boxes
 	bool savetoWallet = false;
 	bool requestPassword = true;
 
@@ -112,6 +114,10 @@ KFritzWindow::KFritzWindow()
 	setupActions();
 
 	setupGUI();
+	KXmlGuiWindow::stateChanged("NoEdit");
+
+	// remove handbook menu entry
+	actionCollection()->action("help_contents")->setVisible(false);
 }
 
 KFritzWindow::~KFritzWindow()
@@ -149,7 +155,7 @@ void KFritzWindow::HandleConnect(int connId __attribute__((unused)))
 {
 	if (notification)
 		notification->close();
-	emit signalNotification("callConnected", "Call connected.", false);
+	emit signalNotification("callConnected", i18n("Call connected."), false);
 }
 
 void KFritzWindow::HandleDisconnect(int connId __attribute__((unused)), std::string duration)
@@ -190,6 +196,9 @@ void KFritzWindow::showSettings() {
     QWidget *frameFonbooks = new KSettingsFonbooks(this);
 	confDialog->addPage(frameFonbooks, i18n("Phone books"), "x-office-address-book", i18n("Select phone books to use"));
 
+	QWidget *frameMisc     = new KSettingsMisc(this);
+	confDialog->addPage(frameMisc,     i18n("Other"), "preferences-other", i18n("Configure other settings"));
+
 	connect(confDialog, SIGNAL(settingsChanged(const QString &)), this, SLOT(updateConfiguration(const QString &)));
 
 	confDialog->show();
@@ -200,6 +209,12 @@ void KFritzWindow::showNotificationSettings() {
 }
 
 void KFritzWindow::updateConfiguration(const QString &dialogName __attribute__((unused))) {
+	// stop any pending initialization
+    libFritzInit->terminate();
+    showStatusbarBoxBusy(true);
+    // clean up before changing the configuration
+	fritz::Config::Shutdown();
+
 	libFritzInit->setPassword(fbPassword);
 	libFritzInit->start();
 }
@@ -259,13 +274,70 @@ void KFritzWindow::setupActions() {
 	actionCollection()->addAction("dialNumber", aDialNumber);
 	connect(aDialNumber, SIGNAL(triggered(bool)), this, SLOT(dialNumber()));
 
+	KAction *aCopyNumber = new KAction(this);
+	aCopyNumber->setText(i18n("Copy number to clipboard"));
+	aCopyNumber->setIcon(KIcon("edit-copy"));
+	actionCollection()->addAction("copyNumber", aCopyNumber);
+	connect(aCopyNumber, SIGNAL(triggered(bool)), this, SLOT(copyNumberToClipboard()));
+
+	KAction *aSetDefaultType = new KAction(this);
+	aSetDefaultType->setText(i18n("Set as default"));
+	aSetDefaultType->setIcon(KIcon("favorites"));
+	actionCollection()->addAction("setDefaultType", aSetDefaultType);
+	connect(aSetDefaultType, SIGNAL(triggered(bool)), this, SLOT(setDefaultType()));
+
+	//TODO: Set Important
+
+	KAction *aReload = new KAction(this);
+	aReload->setText(i18n("Reload"));
+	aReload->setIcon(KIcon("view-refresh"));
+	actionCollection()->addAction("reload", aReload);
+	connect(aReload, SIGNAL(triggered(bool)), this, SLOT(reload()));
+
+	KAction *aReconnectISP = new KAction(this);
+	aReconnectISP->setText(i18n("Reconnect to internet"));
+	aReconnectISP->setIcon(KIcon("network-workgroup"));
+	actionCollection()->addAction("reconnectISP", aReconnectISP);
+	connect(aReconnectISP, SIGNAL(triggered(bool)), this, SLOT(reconnectISP()));
+
+	KAction *aGetIP = new KAction(this);
+	aGetIP->setText(i18n("Get current IP address"));
+	aGetIP->setIcon(KIcon("network-server"));
+	actionCollection()->addAction("getIP", aGetIP);
+	connect(aGetIP, SIGNAL(triggered(bool)), this, SLOT(getIP()));
+
+	KAction *aAddEntry = new KAction(this);
+	aAddEntry->setText(i18n("Add"));
+	aAddEntry->setIcon(KIcon("list-add"));
+	aAddEntry->setShortcut(Qt::CTRL + Qt::Key_N);
+	actionCollection()->addAction("addEntry", aAddEntry);
+	connect(aAddEntry, SIGNAL(triggered(bool)), this, SLOT(addEntry()));
+
+	KAction *aDeleteEntry = new KAction(this);
+	aDeleteEntry->setText(i18n("Delete"));
+	aDeleteEntry->setIcon(KIcon("list-remove"));
+	aDeleteEntry->setShortcut(Qt::Key_Delete);
+	actionCollection()->addAction("deleteEntry", aDeleteEntry);
+	connect(aDeleteEntry, SIGNAL(triggered(bool)), this, SLOT(deleteEntry()));
+
+	KAction *aResolveNumber = new KAction(this);
+	aResolveNumber->setText(i18n("Resolve number"));
+	actionCollection()->addAction("resolveNumber", aResolveNumber);
+	connect(aResolveNumber, SIGNAL(triggered(bool)), this, SLOT(resolveNumber()));
+
+	KAction *aSeparator;
+	aSeparator = new KAction(this);
+	aSeparator->setSeparator(true);
+	actionCollection()->addAction("separator1", aSeparator);
+	aSeparator = new KAction(this);
+	aSeparator->setSeparator(true);
+	actionCollection()->addAction("separator2", aSeparator);
+
+	KStandardAction::cut(this, SLOT(cutEntry()), actionCollection());
+	KStandardAction::copy(this, SLOT(copyEntry()), actionCollection());
+	KStandardAction::paste(this, SLOT(pasteEntry()), actionCollection());
 	KStandardAction::quit(kapp, SLOT(quit()), actionCollection());
 
-//TODO: add "find" action
-
-//	KStandardAction::find(this, SLOT(find()), actionCollection());
-//	KStandardAction::findNext(this, SLOT(findNext()), actionCollection());
-//	KStandardAction::findPrev(this, SLOT(findPrev()), actionCollection());
 }
 
 void KFritzWindow::initIndicator() {
@@ -294,15 +366,11 @@ bool KFritzWindow::queryClose() {
 	return false;
 }
 
-
 void KFritzWindow::updateMissedCallsIndicator() {
 	fritz::CallList *callList = fritz::CallList::getCallList(false);
-	size_t missedCallCount = 0;
-	if (callList) {
-		for (size_t pos = 0; pos < callList->GetSize(fritz::CallEntry::MISSED); pos++)
-			if (KSettings::lastKnownMissedCall() < callList->RetrieveEntry(fritz::CallEntry::MISSED, pos)->timestamp)
-				missedCallCount++;
-	}
+	if (!callList)
+		return;
+	size_t missedCallCount = callList->MissedCalls(KSettings::lastKnownMissedCall());
 #ifdef INDICATEQT_FOUND
 	missedCallsIndicator->setCountProperty(missedCallCount);
 	if (missedCallCount == 0)
@@ -335,43 +403,72 @@ void KFritzWindow::updateMainWidgets(bool b)
 	// init call list, add to tabWidget
 	KCalllistModel *modelCalllist;
 	modelCalllist = new KCalllistModel();
-
 	connect(modelCalllist, SIGNAL(updated()), this, SLOT(updateMissedCallsIndicator()));
 
 	treeCallList = new QAdaptTreeView(this);
 	treeCallList->setAlternatingRowColors(true);
 	treeCallList->setItemsExpandable(false);
 	treeCallList->setSortingEnabled(true);
+	treeCallList->addAction(actionCollection()->action("edit_copy"));
+	treeCallList->addAction(actionCollection()->action("separator1"));
 	treeCallList->addAction(actionCollection()->action("dialNumber"));
+	treeCallList->addAction(actionCollection()->action("copyNumber"));
+	treeCallList->addAction(actionCollection()->action("resolveNumber"));
 	treeCallList->setContextMenuPolicy(Qt::ActionsContextMenu);
-
-	treeCallList->setModel(modelCalllist);
 	treeCallList->sortByColumn(1, Qt::DescendingOrder); //sort by Date
 
-	tabWidget->insertTab(0, treeCallList, KIcon("application-x-gnumeric"), 	i18n("Call history"));
+	// init proxy class for filtering
+	KCalllistProxyModel *proxyModelCalllist = new KCalllistProxyModel(this);
+	proxyModelCalllist->setSourceModel(modelCalllist);
+	treeCallList->setModel(proxyModelCalllist);
+
+	// search line widget
+	KFilterProxySearchLine *search = new KFilterProxySearchLine(this);
+	search->setProxy(proxyModelCalllist);
+
+	// setup final calllist widget
+	ContainerWidget *calllistContainer = new ContainerWidget(this, treeCallList, proxyModelCalllist);
+	new QVBoxLayout(calllistContainer);
+	calllistContainer->layout()->addWidget(search);
+	calllistContainer->layout()->addWidget(treeCallList);
+	tabWidget->insertTab(0, calllistContainer, KIcon("application-x-gnumeric"), 	i18n("Call history"));
 
 	// init fonbooks, add to tabWidget
     fritz::FonbookManager *fm = fritz::FonbookManager::GetFonbookManager();
-    std::string first = fm->GetTechId(); //todo: no fb configured
-    do {
+    std::string first = fm->GetTechId();
+    if (first.length()) {
+    	do {
+    		KFonbookModel *modelFonbook = new KFonbookModel(fm->GetTechId());
 
-		KFonbookModel *modelFonbook = new KFonbookModel(fm->GetTechId());
+    		QAdaptTreeView *treeFonbook = new QAdaptTreeView(this);
+    		treeFonbook->setAlternatingRowColors(true);
+    		treeFonbook->setItemsExpandable(true);
+    		treeFonbook->setSortingEnabled(true);
+    		treeFonbook->setModel(modelFonbook);
+    		treeFonbook->sortByColumn(0, Qt::AscendingOrder); //sort by Name
+    		treeFonbook->addAction(actionCollection()->action("edit_cut"));
+    		treeFonbook->addAction(actionCollection()->action("edit_copy"));
+    		treeFonbook->addAction(actionCollection()->action("edit_paste"));
+    		treeFonbook->addAction(actionCollection()->action("separator1"));
+    		treeFonbook->addAction(actionCollection()->action("addEntry"));
+    		treeFonbook->addAction(actionCollection()->action("deleteEntry"));
+    		treeFonbook->addAction(actionCollection()->action("separator2"));
+    		treeFonbook->addAction(actionCollection()->action("dialNumber"));
+    		treeFonbook->addAction(actionCollection()->action("copyNumber"));
+    		treeFonbook->addAction(actionCollection()->action("setDefaultType"));
+    		treeFonbook->setContextMenuPolicy(Qt::ActionsContextMenu);
 
-		QAdaptTreeView *treeFonbook = new QAdaptTreeView(this);
-		treeFonbook->setAlternatingRowColors(true);
-		treeFonbook->setItemsExpandable(false);
-		treeFonbook->setSortingEnabled(true);
-		treeFonbook->setModel(modelFonbook);
-		treeFonbook->sortByColumn(0, Qt::AscendingOrder); //sort by Name
-		treeFonbook->addAction(actionCollection()->action("dialNumber"));
-		treeFonbook->setContextMenuPolicy(Qt::ActionsContextMenu);
-
-
-		tabWidget->insertTab(0, treeFonbook,  KIcon("x-office-address-book"), 	i18n(fm->GetTitle().c_str()));
-
-		fm->NextFonbook();
-	} while( first != fm->GetTechId() );
+    		ContainerWidget *fonbookContainer = new ContainerWidget(this, treeFonbook, modelFonbook);
+    		new QVBoxLayout(fonbookContainer);
+    		fonbookContainer->layout()->addWidget(treeFonbook);
+    		tabWidget->insertTab(0, fonbookContainer,  KIcon("x-office-address-book"), 	i18n(fm->GetTitle().c_str()));
 
+    		fm->NextFonbook();
+    	} while( first != fm->GetTechId() );
+    }
+    connect(tabWidget, SIGNAL(currentChanged(int)), this, SLOT(updateActionProperties(int)));
+    connect(tabWidget, SIGNAL(currentChanged(int)), statusBar(), SLOT(clearMessage()));
+    updateActionProperties(tabWidget->currentIndex());
 }
 
 void KFritzWindow::find() {
@@ -411,20 +508,138 @@ void KFritzWindow::showLog() {
 	logDialog->show();
 }
 
-void KFritzWindow::dialNumber() {
-	QAdaptTreeView *currentView = static_cast<QAdaptTreeView *>(tabWidget->currentWidget());
-	std::string currentNumber;
-	if (currentView)
-		currentNumber = currentView->currentNumber();
-	if (currentNumber.length() == 0) {
-		KMessageBox::sorry(this, i18n("Can not dial without a number.\nPlease select an entry in a phone book or the call history which contains a number or name and try again."), i18n("Dialing not possible"));
-		return;
+std::string KFritzWindow::getCurrentNumber() {
+	ContainerWidget *container = static_cast<ContainerWidget *>(tabWidget->currentWidget());
+	QAdaptTreeView *treeView = container->getTreeView();
+	if (container->isFonbook()) {
+		return container->getFonbookModel()->number(treeView->currentIndex());
+	} else if (container->isCalllist()) {
+		return container->getCalllistModel()->number(treeView->currentIndex());
 	}
-	//setProgressIndicator(i18n("Dialing %1", currentNumber.c_str())); //TODO: doesn't work yet (dialNumber blocks GUI thread)
-	DBG("Dialing number = " << currentNumber);
+	return "";
+}
+
+void KFritzWindow::dialNumber() {
+	DialDialog d(this, getCurrentNumber());
+	d.exec();
+}
+
+void KFritzWindow::copyNumberToClipboard() {
+	KApplication::kApplication()->clipboard()->setText(getCurrentNumber().c_str());
+}
+
+void KFritzWindow::setDefaultType() {
+	ContainerWidget *container = static_cast<ContainerWidget *>(tabWidget->currentWidget());
+	QAdaptTreeView *treeView = container->getTreeView();
+	if (container->isFonbook()) {
+		container->getFonbookModel()->setDefaultType(treeView->currentIndex());
+	}
+}
+
+void KFritzWindow::reload() {
+	updateConfiguration();
+}
+
+void KFritzWindow::reconnectISP() {
 	fritz::FritzClient fc;
-	fc.InitCall(currentNumber);
-	//setProgressIndicator();
+	fc.reconnectISP();
+	KMessageBox::information(this, i18n("Reconnect initiated."));
+}
+
+void KFritzWindow::getIP() {
+	fritz::FritzClient fc;
+	KMessageBox::information(this, i18n("Current IP address is: %1", fc.getCurrentIP().c_str()));
+}
+
+void KFritzWindow::addEntry(fritz::FonbookEntry *fe) {
+	ContainerWidget *container = static_cast<ContainerWidget *>(tabWidget->currentWidget());
+	QAdaptTreeView *treeView = container->getTreeView();
+	if (container->isFonbook()) {
+		KFonbookModel *model = container->getFonbookModel();
+		if (fe)
+			model->insertFonbookEntry(treeView->currentIndex(), *fe);
+		else
+			model->insertRows(container->getTreeView()->currentIndex().row(), 1, QModelIndex());
+		treeView->scrollTo(container->getTreeView()->currentIndex());
+		treeView->setCurrentIndex(model->index(treeView->currentIndex().row()-1,0,QModelIndex()));
+	}
+}
+
+void KFritzWindow::deleteEntry() {
+	ContainerWidget *container = static_cast<ContainerWidget *>(tabWidget->currentWidget());
+	QAdaptTreeView *treeView = container->getTreeView();
+	if (container->isFonbook()) {
+		KFonbookModel *model = container->getFonbookModel();
+		size_t newRow = treeView->currentIndex().row();
+		if (treeView->currentIndex().row() == model->rowCount()-1)
+			newRow--;
+		QModelIndex index = model->index(newRow, treeView->currentIndex().column());
+		model->removeRows(treeView->currentIndex().row(), 1, QModelIndex());
+		treeView->setCurrentIndex(index);
+	}
+}
+
+void KFritzWindow::cutEntry() {
+	copyEntry();
+	deleteEntry();
+}
+
+void KFritzWindow::copyEntry() {
+	ContainerWidget *container = static_cast<ContainerWidget *>(tabWidget->currentWidget());
+	QAdaptTreeView *treeView = container->getTreeView();
+
+	if (container->isFonbook()) {
+		KFonbookModel *model = container->getFonbookModel();
+		const fritz::FonbookEntry *fe = model->retrieveFonbookEntry(treeView->currentIndex());
+		QMimeData* mimeData = new MimeFonbookEntry(*fe);
+		QApplication::clipboard()->setMimeData(mimeData);
+	}
+	if (container->isCalllist()) {
+		KCalllistProxyModel *model = container->getCalllistModel();
+		const fritz::CallEntry *ce = model->retrieveCallEntry(treeView->currentIndex());
+		fritz::FonbookEntry fe(ce->remoteName);
+		fe.AddNumber(ce->remoteNumber, fritz::FonbookEntry::TYPE_NONE);
+		QMimeData* mimeData = new MimeFonbookEntry(fe);
+		QApplication::clipboard()->setMimeData(mimeData);
+	}
+}
+
+void KFritzWindow::pasteEntry() {
+	const QMimeData *mime = QApplication::clipboard()->mimeData();
+	if (mime) {
+		const MimeFonbookEntry *mimeFonbookEntry = qobject_cast<const MimeFonbookEntry *>(mime);
+		if (mimeFonbookEntry) {
+			fritz::FonbookEntry *fe = mimeFonbookEntry->retrieveFonbookEntry();
+			if (fe)
+				addEntry(fe);
+		}
+	}
+}
+
+void KFritzWindow::resolveNumber() {
+	ContainerWidget *container = static_cast<ContainerWidget *>(tabWidget->currentWidget());
+	QAdaptTreeView *treeView = container->getTreeView();
+	if (container->isCalllist()) {
+		std::string currentNumber = getCurrentNumber();
+		fritz::Fonbook::sResolveResult result = fritz::FonbookManager::GetFonbook()->ResolveToName(currentNumber);
+		if (!result.name.compare(currentNumber)) {
+			statusBar()->showMessage(i18n("%1 did not resolve.", QString(currentNumber.c_str())), 0);
+		} else {
+			fritz::CallEntry *entry = container->getCalllistModel()->retrieveCallEntry(treeView->currentIndex());
+			entry->remoteName = result.name;
+			//TODO: change all calllist entries with this number
+		    statusBar()->showMessage(i18n("%1 resolves to \"%2\".", QString(currentNumber.c_str()), QString(result.name.c_str())), 0);
+		}
+	}
+}
+
+void KFritzWindow::updateActionProperties(int tabIndex __attribute__((unused))) {
+	KXmlGuiWindow::stateChanged("NoEdit");
+	ContainerWidget *container = static_cast<ContainerWidget *>(tabWidget->currentWidget());
+	if (container->isFonbook()) {
+		if (container->getFonbookModel()->flags(QModelIndex()) & QFlag(Qt::ItemIsEditable))
+			KXmlGuiWindow::stateChanged("WriteableFB");
+	}
 }
 
 void KFritzWindow::setProgressIndicator(QString message) {
@@ -447,6 +662,7 @@ void KFritzWindow::setProgressIndicator(QString message) {
 		progressIndicator->layout()->addWidget(label);
 		progressIndicator->layout()->addWidget(bar);
 		progressIndicator->layout()->setMargin(0);
-		statusBar()->insertPermanentWidget(1, progressIndicator);
+		statusBar()->insertPermanentWidget(0, progressIndicator);
 	}
 }
+
