@@ -21,10 +21,10 @@
 
 #include "QAdaptTreeView.h"
 #include "KFritzModel.h"
+#include "KCalllistProxyModel.h"
 
 QAdaptTreeView::QAdaptTreeView(QWidget *parent)
 :QTreeView(parent) {
-//	connect(model, SIGNAL(dataChanged(const QModelIndex &, const QModelIndex& )),			SLOT(adaptColumns(const QModelIndex &, const QModelIndex&)));
 }
 
 QAdaptTreeView::~QAdaptTreeView() {
@@ -33,6 +33,7 @@ QAdaptTreeView::~QAdaptTreeView() {
 
 void QAdaptTreeView::reset() {
 	QTreeView::reset();
+	expandAll();
 	adaptColumns();
 }
 
@@ -41,10 +42,3 @@ void QAdaptTreeView::adaptColumns() {
     for (int col=0; col < model()->columnCount(QModelIndex()); col++)
            resizeColumnToContents(col);
 }
-
-std::string QAdaptTreeView::currentNumber() {
-	if (currentIndex().isValid())
-		return static_cast<KFritzModel *>(model())->number(currentIndex());
-	else
-		return "";
-}
